package javax.management;

import java.util.Objects;
import java.util.Arrays;
import com.sun.jmx.mbeanserver.Introspector;
import java.lang.reflect.Constructor;

public class MBeanConstructorInfo extends MBeanFeatureInfo implements Cloneable
{
    static final long serialVersionUID = 4433990064191844427L;
    static final MBeanConstructorInfo[] NO_CONSTRUCTORS;
    private final transient boolean arrayGettersSafe;
    private final MBeanParameterInfo[] signature;
    
    public MBeanConstructorInfo(String s, Constructor<?> elmt) {
        this(elmt.getName(), s, constructorSignature(elmt), Introspector.descriptorForElement(elmt));
    }
    
    public MBeanConstructorInfo(String s, String s2, MBeanParameterInfo[] array) {
        this(s, s2, array, null);
    }
    
    public MBeanConstructorInfo(String s, String s2, MBeanParameterInfo[] no_PARAMS, Descriptor descriptor) {
        super(s, s2, descriptor);
        if (no_PARAMS == null || no_PARAMS.length == 0) {
            no_PARAMS = MBeanParameterInfo.NO_PARAMS;
        }
        else {
            no_PARAMS = (MBeanParameterInfo[])no_PARAMS.clone();
        }
        this.signature = no_PARAMS;
        this.arrayGettersSafe = MBeanInfo.arrayGettersSafe(this.getClass(), MBeanConstructorInfo.class);
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException ex) {
            return null;
        }
    }
    
    public MBeanParameterInfo[] getSignature() {
        if (this.signature.length == 0) {
            return this.signature;
        }
        return (MBeanParameterInfo[])this.signature.clone();
    }
    
    private MBeanParameterInfo[] fastGetSignature() {
        if (this.arrayGettersSafe) {
            return this.signature;
        }
        return this.getSignature();
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + "[description=" + this.getDescription() + ", name=" + this.getName() + ", signature=" + Arrays.asList(this.fastGetSignature()) + ", descriptor=" + this.getDescriptor() + "]";
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof MBeanConstructorInfo)) {
            return false;
        }
        MBeanConstructorInfo mBeanConstructorInfo = (MBeanConstructorInfo)o;
        return Objects.equals(mBeanConstructorInfo.getName(), this.getName()) && Objects.equals(mBeanConstructorInfo.getDescription(), this.getDescription()) && Arrays.equals(mBeanConstructorInfo.fastGetSignature(), this.fastGetSignature()) && Objects.equals(mBeanConstructorInfo.getDescriptor(), this.getDescriptor());
    }
    
    @Override
    public int hashCode() {
        return Objects.hash(new Object[] { this.getName() }) ^ Arrays.hashCode(this.fastGetSignature());
    }
    
    private static MBeanParameterInfo[] constructorSignature(Constructor<?> constructor) {
        return MBeanOperationInfo.parameters(constructor.getParameterTypes(), constructor.getParameterAnnotations());
    }
    
    static {
        NO_CONSTRUCTORS = new MBeanConstructorInfo[0];
    }
}
